import es
import time
import os
import popuplib
import cfglib
import langlib
import esamlib
import installib
import services
import cmdlib

'''
    Uptime Changelog

    V1.1 (CURRENT)
        added cfglib.addonINI and langlib support
        added autoupdate feature
        added auth service support, also supports a basic adminlist via cvar
        added cmdlib support, rm player_say event
    V1.0
        basic release
        logs uptimes and other info
'''

#plugin information
info = es.AddonInfo() 
info.name     = 'Uptime' 
info.version  = '1.1' 
info.url      = 'http://addons.eventscripts.com/addons/view/uptime' 
info.basename = 'uptime' 
info.author   = 'Errant'


# V1.1 converted STRINGS to use cfglib.AddonINI and langlib
strings = cfglib.AddonINI(es.getAddonPath("uptime") + "/strings.ini")
ini.setInitialComments(['Strings File for Uptime.py','Please submit any additions to the forum thread'])

strings.addGroup('total')
strings.addValueToGroup('total', 'en', "Server has been up for $time")

strings.addGroup('map')
strings.addValueToGroup('map', 'en', "Map has been running for $time")

strings.addGroup('total_players')
strings.addValueToGroup('total_players', 'en', "$players unique players since server start")

strings.addGroup('map_players')
strings.addValueToGroup('map_players', 'en', "$players unique players this map")

strings.write()
        
text = langlib.Strings(strings)

# CFG SETUP
config = cfglib.AddonCFG(es.getAddonPath("uptime") + "/uptime.cfg")
config.text("******************************")
config.text("  UPTIME SETTINGS")
config.text("******************************")
 
date_format         = config.cvar("uptime_date_format",     0, "Set date format (1 = US format, 0 = EU format)")
log_on              = config.cvar("uptime_log",     1, "Log times to file on map start (1=ON, 0=OFF)")
show_round_start    = config.cvar("uptime_rnd_start",     1, "Show uptimes publicly on round start  (1=ON, 0=OFF)")
show_as_popup       = config.cvar("uptime_use_popup",     1, "Show uptimes via popup  (1=ON, 0=OFF)")
auto_update         = config.cvar("uptime_auto_update",     1, "Auto update on map start, if available  (1=ON, 0=OFF)")
admin_only          = config.cvar("uptime_admins_only",     0, "Only show the uptime info to admins  (1=ON, 0=OFF)")
admins              = config.cvar("uptime_admins",     '', "Uptime adminlist")
 
config.write() # Writes the .cfg to file

# vars
last_update = 0
uptime = {}

# auth
if services.isRegistered('auth'):
    auth = services.use('auth')
    auth.registerCapability('uptime_admin',1)
else:
    auth = None
    
def isAuthorised(id):
    # if admin_only == 0 then anyone can see it :)
    if int(admin_only) == 0:
        return True
    if auth:
        # if auth is available then try that
        if auth.isUseridAuthorized(id,'uptime_admin') == 1:
            return True
    else:
        # basic admin check
        if es.getplayersteamid(id) in admins.split(';'):
            return True
    # if we get to here we are NOT authed
    return False

def autoUpdate():
    # NEW in V1.1
    # auto update on map start
    global last_update
    # update once a day
    if (timeInt() - last_update) > 86400:
        esam = esamlib.getAddon('uptime')
        # check versions
        if str(esam.currentversion) != info.version:
            # grab an installlib instance and force an update
            installer = installlib.getInstaller('uptime')
            installer.update(True)
        last_update = timeInt()
    
def timeInt():
    # return a Epoc seconds integer
    return int(time.time())
    
def strftime(timeS=False):
    # strftime wrapper method
    if int(date_format) == 1:
        format = '[%m/%d/%y %H:%M:%S]'
    else:
        format = '[%d/%m/%y %H:%M:%S]'
    if timeS:
        return time.strftime(format,time.localtime(timeS))
    else:
        return time.strftime(format)
        
def humanize_time(secs):
    # turns the time difference into days, hrs, minutes, secs
    mins, secs = divmod(secs, 60)
    hours, mins = divmod(mins, 60)
    days, hours = divmod(hours, 24)
    return '%02dd %02dh %02dm %02ds' % (days,hours, mins, secs)
   
def show_time(stype="total",user=False,ret=False):
    # show time..
    # self explanatory
    s_timed = humanize_time(timeInt() - uptime[stype])
    if ret == True:
        return s_timed
    if user:
        es.tell(user,'#green',text(stype, {'time':s_timed}, playerlib.Player(user).get('lang') ) )
    else:
        es.msg('#green',text(stype, {'time':s_timed}) )
        
def show_players(stype="total",user=False,ret=False):
    # show player data
    # self explanatory
    if ret == True:
        return len(uptime["%s_players" % stype])
    if user:
        es.tell(user,'#green',text("%s_players" % stype, {'players':len(uptime["%s_players" % stype]}, playerlib.Player(user).get('lang') ) ) )
    else:
        es.msg('#green',text("%s_players" % stype, {'players':len(uptime["%s_players" % stype])}))      
        
def load():
    # reset on load :)
    uptime['total'] = timeInt()
    uptime['map'] = timeInt()
    uptime['total_players'] = []
    uptime['map_players'] = []
    # cfg exec
    config.execute()
    # commands
    cmdlib.registerClientCommand('uptime', uptime_cmd, 'View Server Uptime Information', 'uptime_admin', 'ADMIN', uptime_cmd_noauth) 
    cmdlib.registerSayCommand('uptime', uptime_cmd, 'View Server Uptime Information', 'uptime_admin', 'ADMIN', uptime_cmd_noauth)

def uptime_cmd(userid, args):
    opt = int(show_as_popup)
    if len(args) > 0:
        # we specified something in the cmd
        opts = {'chat':0,'popup':1}
        if str(args[0]) in opts:
            opt = opts[str(args[0])]
    if opt == 1:
        # show popup
        popup = popuplib.create('uptime')
        popup.addline('-- Server Uptime --')
        popup.addline(' by Errant')
        popup.addline('>> Server <<')
        popup.addline('Map Started: %s' % strftime(uptime['map']))
        popup.addline('Map Time:    %s' % show_time('map',ret=True))
        popup.addline('Server Started: %s' % strftime(uptime['total']))
        popup.addline('Total Uptime:   %s' % show_time(ret=True))
        popup.addline('>> Players <<')
        popup.addline('Map Players: %i' % show_players('map',ret=True))
        popup.addline('Total Players:   %i' % show_players(ret=True))
        popup.send(userid)
    else:
        show_time(user=userid)
        show_time('map',userid)
        show_players(user=userid)
        show_time('map',userid)
    
def uptime_cmd_noauth(userid, args):
    # if admin_only == 0 then anyone can see it :)
    if int(admin_only) == 0:
        uptime_cmd(userid, args)
    # if auth is not loaded then try some other checks
    if auth == None:
        # basic cvar check
        if es.getplayersteamid(id) in admins.split(';'):
            uptime_cmd(userid, args)
    
def unload():
   # unreg the commands
   cmdlib.unregisterServerCommand('uptime')

def es_map_start(ev):
    if int(log_on) == 1:
        # log this info to file
        fp = open(os.path.join(es.getAddonPath('uptime'),'uptime.log'),'a')
        s_time = strftime()
        fp.write("%s -- Uptime Log, New Map Loaded --\n" % s_time)
        fp.write("%s Map Started: %s \n" % (s_time,strftime(uptime['map'])))
        fp.write("%s Map Time: %s \n" % (s_time,show_time('map',ret=True)))
        fp.write("%s Server Started: %s \n" % (s_time,strftime(uptime['total'])))
        fp.write("%s Total Uptime: %s \n" % (s_time,show_time('total',ret=True)))
        fp.write("%s Map Players: %s \n" % (s_time,show_players('map',ret=True)))
        fp.write("%s Total Players: %s \n" % (s_time,show_players(ret=True)))
        fp.write("%s -- End Update --\n" % s_time)
        fp.close()
    # reset map time & player counts
    uptime['map'] = timeInt()
    uptime['map_players'] = []
    # auto update
    autoUpdate()

def player_activate(ev):    
    if ev['es_steamid'] not in uptime['map_players']:
        uptime['map_players'].append(ev['es_steamid'])
    if ev['es_steamid'] not in uptime['total_players']:
        uptime['total_players'].append(ev['es_steamid'])
        
def round_start(ev):
    # show on round start
    if int(show_round_start) == 1:
        show_time(user=ev['userid'])
        show_time('map',user=ev['userid'])
        show_players(user=ev['userid'])
        show_players('map',user=ev['userid'])